@@ -1,6 +1,6 @@
 '''
 Task Coach - Your friendly task manager
-Copyright (C) 2004-2008 Frank Niessink <frank@niessink.com>
+Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>
 
 Task Coach is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -17,10 +17,18 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 '''
 
 import test
-from taskcoachlib import patterns, command
+from unittests import asserts
+from taskcoachlib import patterns, command, config
+from taskcoachlib.domain import task
 
 
-class CommandTestCase(test.wxTestCase):
+class CommandTestCase(test.wxTestCase, asserts.CommandAssertsMixin):
+    list = []
+    
+    def setUp(self):
+        super(CommandTestCase, self).setUp()
+        task.Task.settings = config.Settings(load=False)
+    
     def tearDown(self):
         super(CommandTestCase, self).tearDown()
         patterns.CommandHistory().clear()
